from typing import Any

def analyze(text: str) -> dict[str, Any]: ...
def analyze_with_verification(text: str) -> dict[str, Any]: ...
def check_consistency(responses: list[str]) -> dict[str, Any]: ...
def extract_claims(text: str) -> list[dict[str, Any]]: ...
def extract_entities(text: str) -> list[str]: ...
